@@ -16,7 +16,7 @@ use POE::Component::IRC::Plugin::Whois;
 use Socket;
 use base qw(POE::Component::Pluggable);
 
-our $VERSION = '6.08';
+our $VERSION = '6.22';
 our ($GOT_SSL, $GOT_CLIENT_DNS, $GOT_SOCKET6, $GOT_ZLIB);
 
 BEGIN {
@@ -374,14 +374,21 @@ sub _sock_up {
     # Remember what IP address we're connected through, for multihomed boxes.
     my $localaddr;
     if ($GOT_SOCKET6) {
-        eval { $localaddr = (unpack_sockaddr_in6( getsockname $socket ))[1] };
+        eval { 
+                $localaddr = (unpack_sockaddr_in6( getsockname $socket ))[1];
+                $localaddr = inet_ntop( AF_INET6, $localaddr );
+        };
+    }
+
+    if ( !$localaddr ) {
+        $localaddr = (unpack_sockaddr_in( getsockname $socket ))[1];
+        $localaddr = inet_ntoa($localaddr);
     }
 
-    $localaddr = (unpack_sockaddr_in( getsockname $socket ))[1] if !$localaddr;
     $self->{localaddr} = $localaddr;
 
     if ( $self->{socks_proxy} ) {
-        $self->{socket} = new POE::Wheel::ReadWrite(
+        $self->{socket} = POE::Wheel::ReadWrite->new(
             Handle       => $socket,
             Driver       => POE::Driver::SysRW->new(),
             Filter       => POE::Filter::Stream->new(),
@@ -431,7 +438,7 @@ sub _sock_up {
     }
     
     # Create a new ReadWrite wheel for the connected socket.
-    $self->{socket} = new POE::Wheel::ReadWrite(
+    $self->{socket} = POE::Wheel::ReadWrite->new(
         Handle       => $socket,
         Driver       => POE::Driver::SysRW->new(),
         InputFilter  => $self->{srv_filter},
@@ -640,7 +647,7 @@ sub commasep {
         $args .= ' ' . join ',', @args;
     }
     elsif ( $state eq 'part' and @args > 1 ) {
-        my $chantypes = join('', @{ $self->isupport('CHANTYPES') }) || '#&';
+        my $chantypes = join('', @{ $self->isupport('CHANTYPES') || ['#', '&']});
         my $message;
         if ($args[-1] =~ / +/ || $args[-1] !~ /^[$chantypes]/) {
             $message = pop @args;
@@ -1033,6 +1040,7 @@ sub onlytwoargs {
 sub privandnotice {
     my ($kernel, $state, $to) = @_[KERNEL, STATE, ARG0];
     my $message = join ' ', @_[ARG1 .. $#_];
+    my @messages = split /\n/, $message;
     my $pri = $_[OBJECT]->{IRC_CMDS}->{$state}->[CMD_PRI];
 
     $state =~ s/privmsglo/privmsg/;
@@ -1047,8 +1055,10 @@ sub privandnotice {
 
     $to = join ',', @$to if ref $to eq 'ARRAY';
     $state = uc $state;
-    $state .= " $to :$message";
-    $kernel->yield(sl_prioritized => $pri, $state);
+
+    for my $msg (@messages) {
+        $kernel->yield(sl_prioritized => $pri, "$state $to :$msg");
+    }
     return;
 }
 
@@ -1208,6 +1218,9 @@ sub sl_prioritized {
     if (bytes::length($msg) > $self->{msg_length} - bytes::length($self->nick_name())) {
         $msg = bytes::substr($msg, 0, $self->{msg_length} - bytes::length($self->nick_name()));
     }
+
+    # if we find a newline in the message, take that to be the end of it
+    $msg =~ s/\n.*//gm;
     
     if (@{ $self->{send_queue} }) {
         my $i = @{ $self->{send_queue} };
@@ -1430,7 +1443,7 @@ sub _delay {
     return if !defined $time;
     my $event = shift @{ $arrayref };
     my $alarm_id = $kernel->delay_set( $event => $time => @{ $arrayref } );
-    $self->send_event(irc_delay_set => $alarm_id, $event, @{ $arrayref } ) if $alarm_id;
+    $self->_send_event(irc_delay_set => $alarm_id, $event, @{ $arrayref } ) if $alarm_id;
     return $alarm_id;
 }
 
@@ -1447,7 +1460,7 @@ sub _delay_remove {
     my @old_alarm_list = $kernel->alarm_remove( $alarm_id );
     if (@old_alarm_list) {
         splice @old_alarm_list, 1, 1;
-        $self->send_event(irc_delay_removed => $alarm_id, @old_alarm_list );
+        $self->_send_event(irc_delay_removed => $alarm_id, @old_alarm_list );
         return \@old_alarm_list;
     }
     
@@ -1785,7 +1798,9 @@ L<C<register>|/"register"> and L<C<irc_registered>|/"irc_registered">.
 
 =head2 C<spawn>
 
-Takes a number of arguments, all of which are optional: 
+Takes a number of arguments, all of which are optional. All the options
+below may be supplied to the L<C<connect>|/"connect"> input event as well,
+except for B<'alias'>, B<'options'>, B<'NoDNS'>, and B<'plugin_debug'>.
 
 B<'alias'>, a name (kernel alias) that this instance will be known by;
 
@@ -1855,9 +1870,6 @@ B<'useipv6'>, enable the use of IPv6 for connections.
 C<spawn> will supply reasonable defaults for any of these attributes which are
 missing, so don't feel obliged to write them all out.
 
-All the above options may be supplied to L<C<connect>|/"connect"> input
-event as well.
-
 If the component finds that L<POE::Component::Client::DNS|POE::Component::Client::DNS>
 is installed it will use that to resolve the server name passed. Disable this
 behaviour if you like, by passing: C<< NoDNS => 1 >>.
@@ -1884,7 +1896,7 @@ you do DCC.
 
 SSL support requires L<POE::Component::SSLify|POE::Component::SSLify>, as well
 as an IRC server that supports SSL connections. If you're missing
-POE::Component::SSLify, specifing B<'UseSSL'> will do nothing. The default is to
+POE::Component::SSLify, specifying B<'UseSSL'> will do nothing. The default is to
 not try to use SSL.
 
 Setting B<'Raw'> to true, will enable the component to send
@@ -2176,7 +2188,8 @@ it will be treated as a PART message and dealt with accordingly.
 Sends a public or private message to the nick(s) or channel(s) which
 you specify. Takes 2 arguments: the nick or channel to send a message
 to (use an array reference here to specify multiple recipients), and
-the text of the message to send.
+the text of the message to send. If the message contains newlines, it
+will be split up into multiple messages.
 
 Have a look at the constants in
 L<POE::Component::IRC::Common|POE::Component::IRC::Common> if you would
@@ -2675,7 +2688,7 @@ object.
 
 =head3 C<irc_delay_set>
 
-Emitted on a succesful addition of a delayed event using the
+Emitted on a successful addition of a delayed event using the
 L<C<delay>|/"delay"> method. C<ARG0> will be the alarm_id which can be used
 later with L<C<delay_remove>|/"delay_remove">. Subsequent parameters are
 the arguments that were passed to L<C<delay>|/"delay">.
@@ -2808,7 +2821,7 @@ The only requirement the IRC protocol places on its messages is that they be
 8-bits, and in ASCII. This has resulted in most of the Western world settling
 on ASCII-compatible Latin-1 as a convention. Recently, popular clients have
 begun sending a mixture of Latin-1 and UTF-8 over the wire to allow more
-characters without breaking backward compatability (too much). To decode such
+characters without breaking backward compatibility (too much). To decode such
 messages reliably, see
 L<C<irc_to_utf8>|POE::Component::IRC::Common/"irc_to_utf8"> in
 L<POE::Component::IRC::Common|POE::Component::IRC::Common>.
