@@ -3,7 +3,9 @@ package POE::Component::IRC::Common;
 use strict;
 use warnings;
 
-our $VERSION = '5.18';
+use Encode qw(decode);
+use Encode::Guess;
+our $VERSION = '6.22';
 
 require Exporter;
 use base qw(Exporter);
@@ -12,7 +14,7 @@ our @EXPORT_OK = qw(
     parse_user irc_ip_get_version irc_ip_is_ipv4 irc_ip_is_ipv6 has_color
     has_formatting strip_color strip_formatting NORMAL BOLD UNDERLINE REVERSE
     WHITE BLACK DARK_BLUE DARK_GREEN RED BROWN PURPLE ORANGE YELLOW LIGHT_GREEN
-    TEAL CYAN LIGHT_BLUE MAGENTA DARK_GREY LIGHT_GREY
+    TEAL CYAN LIGHT_BLUE MAGENTA DARK_GREY LIGHT_GREY irc_to_utf8
 );
 our %EXPORT_TAGS = ( ALL => [@EXPORT_OK] );
 
@@ -48,8 +50,9 @@ use constant {
 };
 
 sub u_irc {
-    my $value = shift || return;
-    my $type = shift || 'rfc1459';
+    my ($value, $type) = @_;
+    return if !defined $value;
+    $type = 'rfc1459' if !defined $type;
     $type = lc $type;
 
     if ( $type eq 'ascii' ) {
@@ -66,8 +69,9 @@ sub u_irc {
 }
 
 sub l_irc {
-    my $value = shift || return;
-    my $type = shift || 'rfc1459';
+    my ($value, $type) = @_;
+    return if !defined $value;
+    $type = 'rfc1459' if !defined $type;
     $type = lc $type;
 
     if ( $type eq 'ascii' ) {
@@ -129,7 +133,8 @@ sub parse_mode_line {
 }
 
 sub parse_ban_mask {
-    my $arg = shift || return;
+    my ($arg) = @_;
+    return if !defined $arg;
 
     $arg =~ s/\x2a{2,}/\x2a/g;
     my @ban;
@@ -155,7 +160,7 @@ sub parse_ban_mask {
 sub matches_mask_array {
     my ($masks, $matches, $mapping) = @_;
     
-    return if !$masks || !$matches;
+    return if !defined $masks || !defined $matches;
     return if ref $masks ne 'ARRAY';
     return if ref $matches ne 'ARRAY';
     my $ref = { };
@@ -172,9 +177,9 @@ sub matches_mask_array {
 }
 
 sub matches_mask {
-    my ($mask,$match,$mapping) = @_;
+    my ($mask, $match, $mapping) = @_;
+    return if !defined $mask || !defined $match;
 
-    return if !$mask || !$match;
     $mask = parse_ban_mask($mask);
     $mask =~ s/\x2A+/\x2A/g;
 
@@ -188,26 +193,31 @@ sub matches_mask {
 }
 
 sub parse_user {
-    my $user = shift || return;
+    my ($user) = @_;
+    return if !defined $user;
+
     my ($n, $u, $h) = split /[!@]/, $user;
     return ($n, $u, $h) if wantarray();
     return $n;
 }
 
 sub has_color {
-    my $string = shift;
+    my ($string) = @_;
+    return if !defined $string;
     return 1 if $string =~ /[\x03\x04]/;
     return;
 }
 
 sub has_formatting {
-    my $string = shift;
+    my ($string) = @_;
+    return if !defined $string;
     return 1 if $string =~/[\x02\x1f\x16\x1d\x11]/;
     return;
 }
 
 sub strip_color {
-    my $string = shift;
+    my ($string) = @_;
+    return if !defined $string;
     
     # mIRC colors
     $string =~ s/\x03(?:,\d{1,2}|\d{1,2}(?:,\d{1,2})?)?//g;
@@ -220,18 +230,26 @@ sub strip_color {
 }
 
 sub strip_formatting {
-    my $string = shift;
+    my ($string) = @_;
+    return if !defined $string;
     $string =~ s/[\x0f\x02\x1f\x16\x1d\x11]//g;
     return $string;
 }
 
+sub irc_to_utf8 {
+    my ($line) = @_;
+    my $utf8 = guess_encoding($line, 'utf8');
+    return ref $utf8 ? decode('utf8', $line) : decode('cp1252', $line);
+}
+
 #------------------------------------------------------------------------------
 # Subroutine ip_get_version
 # Purpose           : Get an IP version
 # Params            : IP address
 # Returns           : 4, 6, 0(don't know)
 sub irc_ip_get_version {
-    my $ip = shift || return;
+    my ($ip) = @_;
+    return if !defined $ip;
 
     # If the address does not contain any ':', maybe it's IPv4
     return 4 if $ip !~ /:/ && irc_ip_is_ipv4($ip);
@@ -248,7 +266,8 @@ sub irc_ip_get_version {
 # Params            : IP address
 # Returns           : 1 (yes) or 0 (no)
 sub irc_ip_is_ipv4 {
-    my $ip = shift || return;
+    my ($ip) = @_;
+    return if !defined $ip;
 
     # Check for invalid chars
     if ($ip !~ /^[\d\.]+$/) {
@@ -306,7 +325,8 @@ sub irc_ip_is_ipv4 {
 # Params            : IP address
 # Returns           : 1 (yes) or 0 (no)
 sub irc_ip_is_ipv6 {
-    my $ip = shift || return;
+    my ($ip) = @_;
+    return if !defined $ip;
 
     # Count octets
     my $n = ($ip =~ tr/:/:/);
@@ -363,8 +383,8 @@ __END__
 
 =head1 NAME
 
-POE::Component::IRC::Common - provides a set of common functions for the
-L<POE::Component::IRC|POE::Component::IRC> suite.
+POE::Component::IRC::Common - Provides a set of common functions for the
+L<POE::Component::IRC|POE::Component::IRC> suite
 
 =head1 SYNOPSIS
 
@@ -386,7 +406,7 @@ L<POE::Component::IRC|POE::Component::IRC> suite.
  if ( matches_mask( $full_banmask, 'stalin!joe@kremlin.ru' ) ) {
      print "EEK!";
  }
-  
+
  if ( has_color($message) ) {
     print 'COLOR CODE ALERT!";
  }
@@ -474,8 +494,9 @@ Takes a list representing an IRC mode line. Returns a hashref. If the modeline
 couldn't be parsed the hashref will be empty. On success the following keys
 will be available in the hashref:
 
- 'modes', an arrayref of normalised modes;
- 'args', an arrayref of applicable arguments to the modes;
+B<'modes'>, an arrayref of normalised modes;
+
+B<'args'>, an arrayref of applicable arguments to the modes;
 
 Example:
 
@@ -504,7 +525,7 @@ Takes two parameters, a string representing an IRC mask ( it'll be processed
 with parse_ban_mask() to ensure that it is normalised ) and something to match
 against the IRC mask, such as a nick!user@hostname string. Returns a true
 value if they match, a false value otherwise. Optionally, one may pass the
-casemapping (see L<C<u_irc()>|/"u_irc">), as this function uses C<u_irc()>
+casemapping (see L<C<u_irc>|/"u_irc">), as this function uses C<u_irc>
 internally.
 
 =head2 C<matches_mask_array>
@@ -513,8 +534,8 @@ Takes two array references, the first being a list of strings representing
 IRC masks, the second a list of somethings to test against the masks. Returns
 an empty hashref if there are no matches. Otherwise, the keys will be the
 masks matched, each value being an arrayref of the strings that matched it.
-Optionally, one may pass the casemapping (see L<C<u_irc()>|/"u_irc">), as
-this function uses C<u_irc()> internally.
+Optionally, one may pass the casemapping (see L<C<u_irc>|/"u_irc">), as
+this function uses C<u_irc> internally.
 
 =head2 C<parse_user>
 
@@ -536,7 +557,7 @@ formatting codes, 0 otherwise.
 
 =head2 C<strip_color>
 
-Takes one paramter, a string of IRC text. Returns the string stripped of all
+Takes one parameter, a string of IRC text. Returns the string stripped of all
 IRC color codes. Due to the fact that both color and formatting codes can
 be cancelled with the same character, this might strip more than you hoped for
 if the string contains both color and formatting codes. Stripping both will
@@ -544,12 +565,45 @@ always do what you expect it to.
 
 =head2 C<strip_formatting>
 
-Takes one paramter, a string of IRC text. Returns the string stripped of all
+Takes one parameter, a string of IRC text. Returns the string stripped of all
 IRC formatting codes. Due to the fact that both color and formatting codes can
 be cancelled with the same character, this might strip more than you hoped for
 if the string contains both color and formatting codes. Stripping both will
 always do what you expect it to.
 
+=head2 C<irc_to_utf8>
+
+The IRC messages you get from L<POE::Component::IRC|POE::Component::IRC> are
+raw byte strings that have no inherent encoding. Most popular clients (mIRC,
+xchat, certain irssi configurations) encode their messages in Microsoft's
+CP1252 encoding (their version of Latin-1) if the message only contains
+characters which fit into Latin-1, otherwise falling back to UTF-8 encoding.
+Writing something like this to a file, terminal, or database is a recipe for
+disaster.
+
+This function takes a byte string (e.g. a message from an
+L<C<irc_public>|POE::Component::IRC/"irc_public"> handler) in "IRC encoding"
+and returns a text string. Since the source encoding might have been UTF-8,
+you should encode/store it in UTF-8 or some other Unicode encoding in your
+file/database/whatever.
+
+ use POE::Component::IRC::Common qw(irc_to_utf8);
+
+ sub irc_public {
+     my ($who, $where, $what) = @_[ARG0..ARG2];
+
+     # not wise, $what is either CP1252 or UTF-8
+     print $what, "\n";
+
+     $what = irc_to_utf8($what);
+
+     # good, $what is always UTF-8
+     print $what, "\n";
+ }
+
+See also L<Encode|Encode>, L<perluniintro>, L<perlunitut>, L<perlunicode>,
+and L<perlunifaq>.
+
 =head2 C<irc_ip_get_version>
 
 Try to guess the IP version of an IP address.
